# Copyright 2019-2020 Turner Broadcasting Inc. / WarnerMedia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from setuptools import setup, find_packages
import os, sys

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
  name='antiope',
  version=os.popen('{} antiope/_version.py'.format(sys.executable)).read().rstrip(),
  author='Chris Farris',
  author_email='chris@room17.com',
  license="Apache License 2.0",
  license_file="LICENSE",
  description='Library for interacting with AWS accounts that are tied to the Antiope framework',
  long_description=long_description,
  long_description_content_type="text/markdown",
  packages=find_packages(),
  py_modules=['antiope'],
  url='https://github.com/WarnerMedia/antiope-aws-module.git',
  python_requires='>=3.6',
  include_package_data=True,
  install_requires=[
    'boto3 >= 1.10.0',
    'botocore >= 1.13.0'
  ]
)
